
/*
 * (a)  (C) 1990 by Adobe Systems Incorporated. All rights reserved.
 *
 * (b)  If this Sample Code is distributed as part of the Display PostScript
 *	System Software Development Kit from Adobe Systems Incorporated,
 *	then this copy is designated as Development Software and its use is
 *	subject to the terms of the License Agreement attached to such Kit.
 *
 * (c)  If this Sample Code is distributed independently, then the following
 *	terms apply:
 *
 * (d)  This file may be freely copied and redistributed as long as:
 *	1) Parts (a), (d), (e) and (f) continue to be included in the file,
 *	2) If the file has been modified in any way, a notice of such
 *      modification is conspicuously indicated.
 *
 * (e)  PostScript, Display PostScript, and Adobe are registered trademarks of
 *	Adobe Systems Incorporated.
 * 
 * (f) THE INFORMATION BELOW IS FURNISHED AS IS, IS SUBJECT TO
 *	CHANGE WITHOUT NOTICE, AND SHOULD NOT BE CONSTRUED
 *	AS A COMMITMENT BY ADOBE SYSTEMS INCORPORATED.
 *	ADOBE SYSTEMS INCORPORATED ASSUMES NO RESPONSIBILITY
 *	OR LIABILITY FOR ANY ERRORS OR INACCURACIES, MAKES NO
 *	WARRANTY OF ANY KIND (EXPRESS, IMPLIED OR STATUTORY)
 *	WITH RESPECT TO THIS INFORMATION, AND EXPRESSLY
 *	DISCLAIMS ANY AND ALL WARRANTIES OF MERCHANTABILITY, 
 *	FITNESS FOR PARTICULAR PURPOSES AND NONINFRINGEMENT
 *	OF THIRD PARTY RIGHTS.
 */

/*
 *	DrawingView.h
 *
 *	This view represents the page that the image is drawn onto. It is
 *	the doc view of the scroll view and can be scrolled.
 *
 *	Version:	2.0
 *	Author:	Ken Fromm
 *	History:
 *			03-07-91		Added this comment.
 */

#import "ScrollApp.h"
#import <appkit/View.h>
#import <appkit/timer.h>

#define REDBOOK		0
#define UPATH		1
#define UCACHE		2

#define MODALTRACE		10

typedef struct _Timing {
	int		num_fills;
	int		num_strokes;
	int		num_subpaths;
	int		num_draws;
	float		drawingtime;
}  Timing;

@interface DrawingView:View
{
	BOOL		displayFields,		/* display the drawing info? */
				eraseFields,		/* erase the drawing info? */
				drawFrame,		/* draw wire frame? */
				drawEverything,	/* draw every graphic? */
				drawCombined,	/* combine each sub path? */
				drawEveryPath;	/* selectively reset each parameter? */

	int			drawManner;		/* draw with red book wraps, user paths */
								/* or user paths with ucache? */

	id			graphicsListId;	/* a list of graphics */

	NXPoint		drawOrigin;

	UPath		*drawUpath;

	Timing		timing_info;
}

+newFrame:(NXRect *) frm;
- free;

- setFieldsMode:(BOOL) mode;
- setTypeOfDrawing:sender;
- setSelectivity:sender;
- setDrawingManner:sender;
- setParameterSetting:sender;
- setStroking:sender;

- setDrawOrigin:(NXPoint *)origin;

- freeGraphics;
- insertList:listId;

- eraseFields:sender;
- displayFields:sender;

- mouseDown:(NXEvent *)event;

- getUpath:(NXRect *) r  fromGraphic:graphicId;
- drawSelf:(NXRect *)r :(int) count;

@end
