
/*
 * (a)  (C) 1990 by Adobe Systems Incorporated. All rights reserved.
 *
 * (b)  If this Sample Code is distributed as part of the Display PostScript
 *	System Software Development Kit from Adobe Systems Incorporated,
 *	then this copy is designated as Development Software and its use is
 *	subject to the terms of the License Agreement attached to such Kit.
 *
 * (c)  If this Sample Code is distributed independently, then the following
 *	terms apply:
 *
 * (d)  This file may be freely copied and redistributed as long as:
 *	1) Parts (a), (d), (e) and (f) continue to be included in the file,
 *	2) If the file has been modified in any way, a notice of such
 *      modification is conspicuously indicated.
 *
 * (e)  PostScript, Display PostScript, and Adobe are registered trademarks of
 *	Adobe Systems Incorporated.
 * 
 * (f) THE INFORMATION BELOW IS FURNISHED AS IS, IS SUBJECT TO
 *	CHANGE WITHOUT NOTICE, AND SHOULD NOT BE CONSTRUED
 *	AS A COMMITMENT BY ADOBE SYSTEMS INCORPORATED.
 *	ADOBE SYSTEMS INCORPORATED ASSUMES NO RESPONSIBILITY
 *	OR LIABILITY FOR ANY ERRORS OR INACCURACIES, MAKES NO
 *	WARRANTY OF ANY KIND (EXPRESS, IMPLIED OR STATUTORY)
 *	WITH RESPECT TO THIS INFORMATION, AND EXPRESSLY
 *	DISCLAIMS ANY AND ALL WARRANTIES OF MERCHANTABILITY, 
 *	FITNESS FOR PARTICULAR PURPOSES AND NONINFRINGEMENT
 *	OF THIRD PARTY RIGHTS.
 */

/*
 *	ScrollApp.h
 *
 *	This class performs some of the global functions necessary to start
 *	the application. The drawing window is created here.
 *
 *	Version:	2.0
 *	Author:	Ken Anderson, Ken Fromm
 *	History:
 *			03-07-91		Added this comment.
 */

#import <appkit/Application.h>
#import <appkit/View.h>

#define FONTSIZE	5
#define MARGIN		3

#define SCROLLVIEW_BORDER NX_NOBORDER

#define  PTS_UPATH_BUFFER		2000
#define  OPS_UPATH_BUFFER		1001

typedef struct _UPath {
	float 	* pts;
	int		num_pts;
	char		*ops;
	int		num_ops;
} UPath;

@interface ScrollApp : Application
{
	id	redrawButton,			/* button for redrawing (also stops drawing) */
		statusmatrixId,		/* ucache status text matrix */
		timingmatrixId,		/* timing text matrix */
		strokingmatrixId,		/* wire frame stroking button matrix */
		parametermatrixId;		/* set each parameter explicity */

	id	menuitemId,			/* Window cell in Activate menu option */
		methodsWindow,		/* needed for modal session */
		drawingviewId,		/* the drawing view */
		docviewId,			/* contains the drawing view, allows zooming*/
		scrollviewId,			/* the scrolling view */
		windowId,			/* the window the views above are in */
		readerId;				/* the object that reads ps files and places */
							/* them in the data structure for display. */

	int	windowBacking;		/* either retained or buffered */

	BOOL	trace,
			runningDrawModal;

	char		*currentfile;		/* complete path name of file */

	UPath	drawBuffer;
}

+ new;

- createWindow:(NXRect *) winRect;

- free;

- setMenuItem:anObject;

- setRedrawButton:anObject;
- redrawButton;
- methodsWindow;

- setStatusMatrix:anObject;
- getStatusMatrix;

- setTimingMatrix:anObject;
- getTimingMatrix;

- setStrokingMatrix:anObject;
- getStrokingMatrix;

- setParameterMatrix:anObject;
- getParameterMatrix;

- setTypeOfDrawing:sender;
- setSelectivity:sender;
- setDrawingManner:sender;
- setParameterSetting:sender;
- setStroking:sender;
- setWindowBacking:sender;

- showWindow:sender;

- setTracing:sender;
- (BOOL) tracing;
- zoom:sender;
- redrawAction:sender;
- redraw:sender;
- displayFields:sender;

- getDrawingView;
- getDocument;
- (UPath *) getUpathBuffer;

- load:sender;

- setName:(const char *)file withParsing:(BOOL) flag;

- windowDidResize:sender;

- (int) runDrawModalSession:(NXModalSession *) session;

@end

