/* File: MessageApp.h - NextStep Interface to Speaker/Mach messages
 *
 * By: Christopher Lane
 * Symbolic Systems Resources Group
 * Knowledge Systems Laboratory
 * Stanford University
 *
 * Date: 24 March 1993
 *
 * Copyright: 1991, 1993 by The Leland Stanford Junior University.  This
 * program may be distributed without restriction for non-commercial use.
 */

#import <appkit/appkit.h>

struct param { id description, type, argument; };

@interface MessageApp:Application
{

	id hostField;
	id hostList;
	id applicationField;
	id applicationList;
	id portField;
	id portList;
	id messageField;
	id messageList;
	id argumentsBox;
	id statusField;
	id messageWindow;
	id stringTable;
	id messageTable;
	id portTable;
	id fileTable;
	id helpScrollView;
	id version;
	
	NXBundle *bundle;
	unsigned int argumentCount;
	struct param params[NX_MAXMSGPARAMS];
}

- appDidInit:sender;

- windowWillClose:sender;

- sendMessage:sender;

- (const char *) loadArguments:(void **) array;

- unloadArguments:(void **) array andUpdate:(BOOL) flag;

- setHostField:object;

- setHostList:object;

- setApplicationField:object;

- setApplicationList:object;

- setPortField:object;

- setPortList:object;

- setMessageField:object;

- setMessageList:object;

- loadPopUpList:popUpList fromNetInfo:(const char *) directory;

- loadPopUpList:popUpList fromDirectory:(const char *) directoryList;

- loadPopUpList:popUpList fromHashTable:hashTable;

- setStatusField:object;

- setArgumentsBox:object;

- setTypeFromMatrix:sender;

- (const char *) getParamTypes:(const char *) name;

- setParamTypes:(const char *) value forMessage:(const char *) name;

- setButton:button fromField:field;

- setField:field fromMatrix:matrix;

- setHostFromField:sender;

- setHostfromMatrix:sender;

- setApplicationFromField:sender;

- setApplicationfromMatrix:sender;

- setPortFromField:sender;

- setPortfromMatrix:sender;

- setMessageFromField:sender;

- setMessagefromMatrix:sender;

- setMessage:(const char *) string;

- (const char *) lookupConstant:(int) value forKey:(const char *) key;

- setVersion:object;

- (int) msgVersion:(const char **) aString ok:(int *) flag;

@end
