/*--------------------------------------------------------------------------
 *
 * 	You may freely copy, distribute, and reuse the code in this example.
 * 	SHL Systemhouse disclaims any warranty of any kind, expressed or  
 *	implied, as to its fitness for any particular use.
 *
 *
 *	ObjectWell
 *
 *	Inherits From:		View
 *
 *	Conforms To:		None
 *
 *	Declared In:		ObjectWell.h
 *
 *------------------------------------------------------------------------*/

#import "ObjectWell.h"



@implementation ObjectWell

/*--------------------------------------------------------------------------
 *	Private
 *------------------------------------------------------------------------*/

- (NXPoint) _centerPointForImage: anImage
{
	NXSize
		imageSize;
	NXRect
		boundsRect;
	
	[self getBounds:&boundsRect];
	[anImage getSize:&imageSize];
	
	if (imageSize.width < NX_WIDTH (&boundsRect))
		{
		NX_X (&boundsRect) += (NX_WIDTH (&boundsRect) - imageSize.width ) / 2.0;
		}
		
	if (imageSize.height < NX_HEIGHT (&boundsRect))
		{
		NX_Y (&boundsRect) += (NX_HEIGHT (&boundsRect) - imageSize.height ) / 2.0;
		}
	
	return boundsRect.origin;
}


- _dragOperationFor: (NXEvent *) originalEvent
	nextEvent: (NXEvent *) nextEvent
{
 	id
		pasteboard;
	NXPoint
		offset = { 0.0, 0.0 };
	NXRect
		iconRect;
	char
		*buffer;
	NSString
		*tmp;
   	
	if (object == nil)
		{
		return self;
		}
	
	iconRect.origin = [self _centerPointForImage:image];
 	pasteboard = [Pasteboard newName:NXDragPboard];
    [pasteboard declareTypes:&NXAsciiPboardType num:1 owner:self];

	if ([object isKindOfClass:[NSString class]])
		{
		tmp = [NSString stringWithFormat:@"\"%@\"", [(NSObject *)object description]];
		}
	else
		{
		tmp = [(NSObject *)object description];
		}
	
	buffer = NXCopyStringBuffer ([(NSString *)tmp cString]);
	[pasteboard writeType:NXAsciiPboardType data:buffer length:strlen(buffer)+1];
    	
	[self dragImage:image
			at:&iconRect.origin
			offset:&offset
			event:originalEvent 
			pasteboard:pasteboard
			source:self
			slideBack:YES];
		
	if (pasteboard)
		{
		[pasteboard free];
		}
	
	return self;
}



/*--------------------------------------------------------------------------
 *	Initialization and Freeing
 *------------------------------------------------------------------------*/

- initFrame: (const NXRect *) frameRect 
{
	[super initFrame:frameRect];
	[self registerForDraggedTypes:&NXAsciiPboardType count:1];
  	return self;
}


- free
{
	[delegate release];
	[object release];
	return [super free];
}



/*--------------------------------------------------------------------------
 *	Accessing the Object
 *------------------------------------------------------------------------*/

- setObject: anObject
{
	[object autorelease];
	object = [anObject retain];
	[self setImage:[NXImage findImageNamed:[object getIcon]]];
	return self;
}


- object
{
	return object;
}


- setImage: anImage
{
	image = anImage;
	[self display];
	return self;
}


- image
{
	return image;
}



/*--------------------------------------------------------------------------
 *	Event Handling
 *------------------------------------------------------------------------*/

- (BOOL) acceptsFirstMouse
{
  	return YES;
}


- mouseDown: (NXEvent *) theEvent
{
	int
		originalEventMask,
		newEventMask;
	NXEvent
		originalEvent = *theEvent,
		nextEvent;

	if (object == nil)
		{
		return self;
		}
	
    [NXApp preventWindowOrdering];
	originalEventMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
	newEventMask = (NX_MOUSEUPMASK | NX_MOUSEDRAGGEDMASK);
	nextEvent = *([NXApp getNextEvent:newEventMask]);

	switch (nextEvent.type)
		{
		case NX_MOUSEDRAGGED:
			[self _dragOperationFor:&originalEvent nextEvent:&nextEvent];
			break;
			
		case NX_MOUSEUP:
		default:
			break;
		}
	
	[window setEventMask:originalEventMask];
	return self;
}



/*--------------------------------------------------------------------------
 *	Delegate
 *------------------------------------------------------------------------*/

- delegate
{
  	return delegate;
}


- setDelegate: anObject 
{
  	[delegate autorelease];
  	delegate = [anObject retain];
  	return self;
}



/*--------------------------------------------------------------------------
 *	Drawing
 *------------------------------------------------------------------------*/

- drawSelf: (const NXRect *) rects
	: (int) rectCount 
{
	NXPoint
		point = [self _centerPointForImage:image];
	
	NXSetColor ([window backgroundColor]);
	NXRectFill (&bounds);
	[image composite:NX_SOVER toPoint:&point];
	return self;
}



/*--------------------------------------------------------------------------
 *	Dragging
 *------------------------------------------------------------------------*/

- (NXDragOperation) draggingSourceOperationMaskForLocal: (BOOL) isLocal;
{
	return NX_DragOperationCopy;
}


- (NXDragOperation) draggingEntered: dragSource
{
    return [self draggingUpdated:dragSource];
}


- (NXDragOperation) draggingUpdated: dragSource
{
	if ([dragSource draggingSource] == self)
		{
		return NX_DragOperationNone;
		}

    return NX_DragOperationCopy;
}


- (BOOL) prepareForDragOperation: dragSource
{
	return YES;
}


- (BOOL) performDragOperation: dragSource
{
	id
		pasteboard,
		tmpObject = nil;
	char
		*data;
	int
		length;

	pasteboard = [dragSource draggingPasteboard];
	[pasteboard readType:NXAsciiPboardType data:&data length:&length];
	
	if (strchr (data, '\t'))
		{
		return NO;
		}
	
	tmpObject = [NSString stringWithCString:data length:length];
	[self setObject:[[tmpObject propertyList] retain]];
	[pasteboard deallocatePasteboardData:data length:length];
	return YES;
}


- notifyDelegateDidDrop: aDragSource
{
	if ([delegate respondsToSelector:@selector (dragSource:didDropOnIconWell:)])
		{
		[delegate dragSource:aDragSource didDropOnIconWell:self];
		}

	return self;
}


- concludeDragOperation: dragSource
{
	[self notifyDelegateDidDrop:dragSource];
	[self update];
	return self;
}


@end
