/*****************************************************************************/
/* SketchView.h								     */
/* interface file for SketchView class of the SketchDemo application         */
/* August 1989 Carl F. Sutter						     */
/*****************************************************************************/

#import <appkit/View.h>
#import <appkit/Bitmap.h>
#import <appkit/OpenPanel.h>
#import <appkit/SavePanel.h>
#import <appkit/Cursor.h>

#define MAXPAGES   20
#define TOOL_RECT   0
#define TOOL_LINE   1
#define TOOL_PENCIL 2
#define UNTITLED "Untitled.sketchD"

@interface SketchView:View
   {
   // internal instance variables    
   int		nCurPage;		/* current page being drawn on */
   Bitmap	*bmpPage[MAXPAGES];	/* bitmaps for the tablet pages */
   int		nTool;			/* current drawing tool */
   char         szFileName[120];	/* cuurent file name */
   OpenPanel	*openReq;		/* open file panel */
   SavePanel	*saveReq;		/* save as file panel */
   Cursor	*curDraw;		/* cursor used when mouse is over sketchpad */
   BOOL		bOpenedWithFile;	/* flag for app launched by opening file */

   // outlet ids
   id		newAlpha;		/* Form for page alpha value */
   id		pageNum;		/* Form for cuurent page number */
   id		numToDraw;		/* TextField for # of pages to draw on flip */
   }

// factory method
+ newFrame:(NXRect *)rect;

// outlet initialization methods
- setNewAlpha:anObject;
- setPageNum:anObject;
- setNumToDraw:anObject;

// App delegate messages
- appDidInit:(id)sender;
- (BOOL)appAcceptsAnotherFile:sender;
- (int)appOpenFile:(const char *)filename type:(const char *)aType;

// actions
- newPage:sender;
- clearPages:sender;
- setTool:sender;
- flipUp:sender;
- flipDown:sender;
- animate:sender;
- openRequest:sender;
- saveAsRequest:sender;
- saveRequest:sender;

// internal methods
- drawSelf:(NXRect *)r :(int) count;
- mouseDown:(NXEvent *)e;
- showError:(char *)errorMessage;
- (BOOL)openSketchPad:(const char *)fileName;
- (BOOL)saveSketchPad:(const char *)fileName;
- resetCursorRects;
- setTitle;
 
@end
