#import <objc/Object.h>
#import <appkit/Application.h>

#import <bsd/sys/param.h>

/* Version 0.9b			Apr-19-92		First Public Release	*/
/* Version 0.95b		Jun-10-92		Minor File path fixes	*/
/* Version 1.00b		Aug-10-92		Cleaned up #imports		*/
/* Version 1.1b			Jan-10-93		Modified for 3.0		*/

// The various shared objects and panels are declared as instance variables
// for several reasons:  first, you can draw connections in IB that way;
// second, it is possible that some other object might want to manipulate
// these objects for their own purposes.														

@interface MultApp: Object
{
	BOOL	haveOpenedDocument;			// Have we opened a document yet?
    id 		plpAccessory;				// accessory view of page layout panel
	id 		infoPanel;					// the Info panel
	id		prefPanel;					// the Preferences panel
	id 		helpPanel;					// the Help panel
	id		pageMargin;					// augmented PageLayout object
	id		stringSet;					// StringSet for localization
	const char *launchDir;				// the app wrapper directory
	char	defaultDir[MAXPATHLEN+1];	// the default directory
	/* Defaults for the application */
	BOOL	saveAll;					// save all upon closing
	BOOL	dumpAll;					// ignore dirty bits upon closing
}

/* Factory Methods */

+ initialize;
+ setDocClass:newDoc;
+ docClass;
+ setDropClass:newDropView;
+ dropClass;

/* Public Methods */

- openFile:(const char *)path file:(const char *)type flag:(BOOL)flag;

- currentDocument;
- setDefaultDir:(const char *)dir;
- (const char *)currentDirectory;
- (const char *)launchDirectory;

- (BOOL)saveAll;
- setSaveAll:(BOOL)value;
- (BOOL)dumpAll;
- setDumpAll:(BOOL)value;

/* Shared Panels Target/Action */

- info:sender;
- pref:sender;
- saveAsPanel:sender;
- pageLayout:sender;
- stringSet:sender;

/* Target/Action Methods */

- new:sender;
- open:sender;
- saveAll:sender;
- print:sender;
- mailToMe:sender;

/* Menu updating method */

- (BOOL)menuItemUpdate:menuCell;
- (BOOL)validateCommand:menuCell;

/* Application Delegate Methods */

- setupPageLayout:(float)lm :(float)rm :(float)tm :(float)bm; 

- app:sender willShowHelpPanel:panel;

- appWillInit:sender;
- appWillTerminate:sender;
- appDidInit:sender;
- (int)appOpenFile:(const char *)path type:(const char *)type;
- (BOOL)appAcceptsAnotherFile:sender;
- (int)app:sender unmounting:(const char *)fullPath;

/* Listener Methods */

//File Manipulation
- (int)msgDirectory:(const char **)fullPath ok:(int *)flag;
- (int)msgFile:(const char **)fullPath ok:(int *)flag;

// Misc
- (int)msgPrint:(const char *)fullPath ok:(int *)flag;
- (int)msgVersion:(const char **)aString ok:(int *)flag;
- (int)msgQuit:(int *)flag;

@end
