/*
 * Copyright 1993 by Cub'x Systemes
 * 
 * All Rights Reserved
 *
 * Permission to use, copy, and distribute this software and its
 * documentation for the  purpose of making new intuitiv'3d modules.
 * This copyright notice must appears in all copies that you distribute.
 * The name of Cub'x Systeme should not be used in advertising or publicity
 * without specific, written prior permission. 
 * CUB'X SYSTEMES DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * CUB'X SYSTEMES BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */
/******************************************************************************* 
 *                                                                             *
 *     Project intuitiv'3d                                                     *
 *     -------------------                                                     *
 *                                                                             *
 *                                                                             *
 *     File: SlideProjector.h                                                  *
 *                                                                             *
 *                                                                             *
 *     Creation date: Tue Jul 27 14:42:12 GMT+0200 1993                        *
 *                                                                             *
 *     Contents: A light object that use the slideprojector shader             *
 *                                                                             *
 *                                                                             *
 *******************************************************************************/

#import <i3d/CX3DLightSource.h>

@interface SlideProjector : CX3DLightSource
{
  float fov;
  RtPoint up;
  char slideName[MAXPATHLEN+1];
  float shadowDetail;
  BOOL useShadow;
  BOOL slideImageIsTiff;

  /*
   *  Instance variables not saved
   */
  char *shadowBuf;       
  BOOL txDone;
  char *txImagePath;
}

/*******************************************************************************
 *                                                                             *
 *                                Class methods                                *
 *                                                                             *
 *******************************************************************************/
+ initialize;
+ (ExportedVariable*) exportedVariable;

/*******************************************************************************
 *                                                                             *
 *                               Instance methods                              *
 *                                                                             *
 *******************************************************************************/
- free;
- init;
- initSelfFromCamera: aCamera;
- copy;
- illuminate;
- preRender: aDisplay;
- postRender: aDisplay;
- render;
- getBoundingBox:(RtBound) aBox;
- getBoundingBoxInWorldSpace:(RtPoint*) p;

/*******************************************************************************
 *                                                                             *
 *                          Parameters and inspectors                          *
 *                                                                             *
 *******************************************************************************/

- setFieldOfView: (float) aValue;
- (float) fieldOfView;
- setUpX:(float) aFloat;
- (float) upX;
- setUpY:(float) aFloat;
- (float) upY;
- setUpZ:(float) aFloat;
- (float) upZ;
- setSlideName:(const char*) aName;
- (const char*) slideName;
- setShadowDetail:(float) aFloat;
- (float) shadowDetail;
- setUseShadow:(BOOL) aFlag;
- (BOOL) useShadow;

- attributeInspClass;
- transformationInspClass;
- shadingInspClass;
- shadersInspClass;
- (AIParameter*) displayParameters;
- (AIParameter*) transformParameters;
- setFromX:(float*) v;
- setFromY:(float*) v;
- setFromZ: (float*)v;
- getFromX:(float*) v;
- getFromY:(float*) v;
- getFromZ:(float*) v;
- setToX:(float*) v;
- setToY:(float*) v;
- setToZ: (float*)v;
- getToX:(float*) v;
- getToY:(float*) v;
- getToZ:(float*) v;
- getMinCoord:(float*)v;
- getMaxCoord:(float*)v;
- getMinAngle:(float*)v;
- getMaxAngle:(float*)v;


/*******************************************************************************
 *                                                                             *
 *                             Reading and writing                             *
 *                                                                             *
 *******************************************************************************/
- awake;
- read:(NXTypedStream*)s;
- write:(NXTypedStream*) s;

@end
