/*
 * Copyright 1993 by Cub'x Systemes
 * 
 * All Rights Reserved
 *
 * Permission to use, copy, and distribute this software and its
 * documentation for the  purpose of making new intuitiv'3d modules.
 * This copyright notice must appears in all copies that you distribute.
 * The name of Cub'x Systeme should not be used in advertising or publicity
 * without specific, written prior permission. 
 * CUB'X SYSTEMES DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * CUB'X SYSTEMES BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

/******************************************************************************* 
 *                                                                             *
 *     Project intuitiv'3d                                                     *
 *     -------------------                                                     *
 *                                                                             *
 *                                                                             *
 *     File: CX3DNode.h                                                        *
 *                                                                             *
 *                                                                             *
 *     Creation date: Mon Jul 26 18:01:17 GMT+0200 1993                        *
 *                                                                             *
 *                                                                             *
 *     Contents: A class to implement a node for 3d objects                    *
 *                                                                             *
 *                                                                             *
 *******************************************************************************/

#import <ri/ri.h>
#import <i3d/NodeView.h>
#import <i3d/i3d_api.h>
#import <i3d/ExportedVariable.h>

@interface N3DNode : NodeView
{
  float scale[3];
  float rotate[3];
  float translate[3];
  float oldValue[3];
  RtMatrix transform;
  RtMatrix rotateMatrix;
  RtMatrix oldRotateMatrix;
  char  surfaceType;
  int   precision;
  id    shape;
  unsigned char visible;

  /*
   *  Shaders
   */
  id metaShader;

  /*
   *  Bounding box
   */
  RtBound bbox;
  BOOL shouldDisplayAsBBox;
  unsigned long totalNumberOfObject;
  BOOL transformDirty;
  int  lastUsedInsp;
}

+ (ExportedVariable*) exportedVariable;
- init;
- copy;
- imageCopy;
- initShapeFromSelf;
- free;
- freeWithoutShape;
- (BOOL) isSelectable;
- rebuildBoundingBoxes;
- getBoundingBox:(RtBound) aBox;
- setShouldDisplayAsBBox:(BOOL) aFlag;
- (BOOL) shouldDisplayAsBBox;
- updateTotalNumberOfObject;
- (unsigned long) totalNumberOfObject;
- (unsigned long) reconfigureBoundingBoxes:(long) nbmax;
- (BOOL)isAVolume;

/*****************************************************************************
 *                                                                           *
 *  Shading                                                                  *
 *                                                                           *
 *****************************************************************************/

- setColor: (NXColor) aColor;
- setOpacity: (NXColor) aColor;
- setInterpolation: (BOOL) interp;
- setSurface: (int) aSurface;
- setUseOneBBox:(BOOL) aFlag;
- (BOOL) useOneBBox;
- setShadingRate: (float) aValue;
- setPrecision:(int) aPrecision;

/*****************************************************************************
 *                                                                           *
 *              Inspectors                                                   *
 *                                                                           *
 *****************************************************************************/
- attributeInspClass;
- shadingInspClass;

/*****************************************************************************
 *                                                                           *
 * Rendering                                                                 *
 *                                                                           *
 *****************************************************************************/
- preRender: displayer;
- postRender: displayer;
- beforeRender;
- afterRender;

/*****************************************************************************
 *                                                                           *
 * Link                                                                      *
 *                                                                           *
 *****************************************************************************/

- addObject: anObject;
- (BOOL) willAcceptObject: anObject;

/*****************************************************************************
 *                                                                           *
 * Color                                                                     *
 *                                                                           *
 *****************************************************************************/
- acceptColor:(NXColor)co atPoint:(const NXPoint *)aPoint;


/*******************************************************************************
 *                                                                             *
 *                             Reading and writing                             *
 *                                                                             *
 *******************************************************************************/

- rebuildShapeTree: parent;

@end

#define USE_OBJECT_DEF 0
#include <i3d/CX3DObjectAndNodeCommon.h>





