/*
 * Copyright 1993 by Cub'x Systemes
 * 
 * All Rights Reserved
 *
 * Permission to use, copy, and distribute this software and its
 * documentation for the  purpose of making new intuitiv'3d modules.
 * This copyright notice must appears in all copies that you distribute.
 * The name of Cub'x Systeme should not be used in advertising or publicity
 * without specific, written prior permission. 
 * CUB'X SYSTEMES DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * CUB'X SYSTEMES BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

/******************************************************************************* 
 *                                                                             *
 *     Project intuitiv'3d                                                     *
 *     -------------------                                                     *
 *                                                                             *
 *                                                                             *
 *     File: CX3DObjectAndNodeCommon.h                                         *
 *                                                                             *
 *                                                                             *
 *     Creation date: Sat Mar  6 18:50:57 GMT 1993                             *
 *                                                                             *
 *                                                                             *
 *     Contents: Common methods for Node and Object                            *
 *                                                                             *
 *                                                                             *
 *******************************************************************************/

#if (USE_OBJECT_DEF==1)
@interface N3DObject(CommonMethods)
#else
@interface N3DNode(CommonMethods)
#endif
#undef USE_OBJECT_DEF

/*****************************************************************************
 *                                                                           *
 * Transformation matrix                                                     *
 *                                                                           *
 *****************************************************************************/
- setTransform:(RtMatrix)m;
- (RtMatrix*)transform;
- resetTransform;

/*******************************************************************************
 *                                                                             *
 *                                Transformation                               *
 *                                                                             *
 *******************************************************************************/
- convertWorldPointsToObject:(RtPoint*) p count:(unsigned int) n;
- convertPointsToWorld:(RtPoint*) p count:(unsigned int) n;
- buildAndGetFinalTransformation:(RtMatrix) m;
- setScaleX: (float) x Y: (float) y Z: (float) z;
- getScaleX: (float*) x Y: (float*) y Z:(float*) z;
- setRotateX: (float) x Y: (float) y Z: (float) z;
- getRotateX: (float*) x Y: (float*) y Z: (float*) z;
- setTranslateX: (float) x Y: (float) y Z: (float) z;
- getTranslateX: (float*) x Y: (float*) y Z: (float*) z;
- saveCurrentTranslation;
- saveCurrentScale;
- saveCurrentRotate;
- getOldValueX: (float*) x Y: (float*) y Z: (float*) z;
- translateOnX:(int) n;
- translateOnY:(int) n;
- translateOnZ:(int) n;
- rotateOnX:(int) n;
- rotateOnY:(int) n;
- rotateOnZ:(int) n;
- setRotateMatrix:(RtMatrix) m;
- getRotateMatrix:(RtMatrix) m;
- saveCurrentRotateMatrix;
- getOldRotateMatrix:(RtMatrix)m;

/*******************************************************************************
 *                                                                             *
 *                                   Shaders                                   *
 *                                                                             *
 *******************************************************************************/
- setColorByReference:(NXColor*) c;
- getColorByReference:(NXColor*) c;
- setOpacityByReference:(NXColor*) c;
- getOpacityByReference:(NXColor*) c;
- (NXColor) color;
- (NXColor) opacity;
- (BOOL) interpolation;
- (int) surface;
- (float) shadingRate;
- setUseLocalShadingRate:(BOOL) aFlag;
- (BOOL) useLocalShadingRate;
- surfaceShader;
- displacementShader;
- areaLightShader;
- interiorShader;
- exteriorShader;
- atmosphereShader;
- setSurfaceShader: anObject;
- setDisplacementShader: anObject;
- setAreaLightShader: anObject;
- setInteriorShader: anObject;
- setExteriorShader: anObject;
- setAtmosphereShader: anObject;
- (BOOL) willAcceptShader: anObject;
- acceptShader: anObject;
- metaShader;
- setTextureCoord: (Point2d*) p1 : (Point2d*) p2 : (Point2d*) p3 : (Point2d*) p4;
- getTextureCoord: (Point2d*) p1 : (Point2d*) p2 : (Point2d*) p3 : (Point2d*) p4;
- setShouldUseTextCoord:(BOOL) aFlag;
- (BOOL) shouldUseTextCoord;
- setShouldReverseOrientation:(BOOL) aFlag;
- (BOOL) shouldReverseOrientation;
- setTextureScale:(float) aValue;
- (float) textureScale;
- setTextureRotation:(float) aValue;
- (float) textureRotation;
- setTextureGrid:(float) aValue;
- (float) textureGrid;
- setShouldUseTextureGrid:(BOOL) aFlag;
- (BOOL) shouldUseTextureGrid;
- metaShaderChange: aMetaShader;
/*******************************************************************************
 *                                                                             *
 *                                   Display                                   *
 *                                                                             *
 *******************************************************************************/
- (int) precision;

/*******************************************************************************
 *                                                                             *
 *                                Bounding boxes                               *
 *                                                                             *
 *******************************************************************************/
- getTransformedBoundingBox:(RtBound) aBox;
- (BOOL) canDisplayAsBoundingBox;


/*******************************************************************************
 *                                                                             *
 *                                  Visibility                                 *
 *                                                                             *
 *******************************************************************************/
- setPhotoVisible:(BOOL) aFlag;
- (BOOL) photoVisible;
- setQuickVisible:(BOOL) aFlag;
- (BOOL) quickVisible;
- setVisible:(BOOL) aFlag;
- (BOOL) visible;

/*******************************************************************************
 *                                                                             *
 *                             Misc and information                            *
 *                                                                             *
 *******************************************************************************/
- (BOOL)isALight;
- (BOOL) viewableInTheBrowser;
- shape;

/*******************************************************************************
 *                                                                             *
 *                                  Inspectors                                 *
 *                                                                             *
 *******************************************************************************/
- transformationInspClass;
- shadersInspClass;
- setLastUsedInsp:(int)flag;
- (int)lastUsedInsp;

/*******************************************************************************
 *                                                                             *
 *                             Reading and writing                             *
 *                                                                             *
 *******************************************************************************/
- (unsigned long) calcSum;
- read:(NXTypedStream*)s;
- write:(NXTypedStream*)s;
- awake;

@end



