/*
 * Copyright 1993 by Cub'x Systemes
 * 
 * All Rights Reserved
 *
 * Permission to use, copy, and distribute this software and its
 * documentation for the  purpose of making new intuitiv'3d modules.
 * This copyright notice must appears in all copies that you distribute.
 * The name of Cub'x Systeme should not be used in advertising or publicity
 * without specific, written prior permission. 
 * CUB'X SYSTEMES DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
 * CUB'X SYSTEMES BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
 * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */
/******************************************************************************* 
 *                                                                             *
 *     Project intuitiv'3d                                                     *
 *     -------------------                                                     *
 *                                                                             *
 *                                                                             *
 *     File: ObjectView.h                                                      *
 *                                                                             *
 *                                                                             *
 *     Creation date: Tue Jul 27 14:25:55 GMT+0200 1993                        *
 *                                                                             *
 *                                                                             *
 *     Contents:An ObjectView is an abstract super-class used to display the   *
 *     instance of a conceptual object as an icon.                             *
 *                                                                             *
 *                                                                             *
 *******************************************************************************/

#import <appkit/appkit.h>

#define ROUND_SIZE    (10)
#define TEXT_HEIGHT   (OV_HEIGHT - 50)
#define OV_WIDTH      (56)
#define OV_HEIGHT     (76)
#define COLOR_ID_SIZE (5.0)

extern void ObjectViewRoundRect(float x1, float x2 , float y1, float y2 );

@interface ObjectView : View
{
  NXSize   imageSize;
  id image;			/* The image used to display the 48x48 icon */
  id textf;			/* The textfield used to display the text   */

  unsigned char selected;       /* True if the ObjectView is selected, False otherwise */
  unsigned char drawnSelected;  /* True if the ObjectView has been drawn selected      */
  unsigned char editOn;         /* True if the text cell is currently being edited           */
  unsigned char shadowOn;       /* Display in shadow mode                              */
  unsigned char deleteNoOk;     /* Whether the object can or can't be deleted			*/
  unsigned char private;        /* Whether the object is or isn't private (no copy)		*/
  unsigned char customIcon;     /* Whether the object doesn't have or has an intuitive'3d icon (need save)  */
  unsigned char haveColorID;    /* Whether the object has or doesn't have a color identifier            */
  NXColor       colorID;        /* The color id                                         */
  id parent;			/* The parent of the object				*/
  id delegate;                  /* The delegate of the object (pathView in dream       */
}

+ textObject;

- init;
- free;
- realFree;
- sizeTo: (NXCoord) w : (NXCoord) h;
- imageSize:(NXSize*) aSize;
- setImageSize: (NXSize*) aSize;
- image;
- setImage: anImage;
- setIconFromMachO:(const char*) name;
- setIconFromFile: (const char*) name;
- setIconByName: (const char*) name;
- setTitle:(const char*) title;
- displayInShadow: (BOOL) flag;
- (BOOL) doesDisplayInShadow;
- titleChange: sender;
- (const char*) title;
- setSelected:(BOOL) aFlag;
- (BOOL) isSelected;
- drawSelf:(const NXRect *)rects :(int)rectCount;
- copy;
- imageCopy;
- doEditingIfNeed:(NXEvent*)anEvent;
- forceEditing;
- textDidEnd:sender endChar:(unsigned short)whyEnd;

- setParent: anObject;
- parent;

- setDeleteOk:(BOOL) flag;
- (BOOL)deleteOk;

- setPrivate:(BOOL) flag;
- (BOOL) isPrivate;

- (BOOL) isDuplicable;
- setDelegate: anObject;
- delegate;

- (BOOL) viewableInTheBrowser;

- (BOOL)shouldDelayWindowOrderingForEvent:(NXEvent *)theEvent;

- setHaveColorID:(BOOL) flag;
- (BOOL) haveColorID;
- setColorID:(NXColor) aColor;
- (NXColor) colorID;
/*****************************************************************************
 *                                                                           *
 *           Method implemented by the parent                                *
 *                                                                           *
 *****************************************************************************/

- (BOOL) sonWillChangeTitle: sender : (const char*) title;

/*****************************************************************************
 *                                                                           *
 *       Method implemented by the delegate                                  *
 *                                                                           *
 *****************************************************************************/

- objectViewTitleChange: anObject;

/*******************************************************************************
 *                                                                             *
 *                             reading and writing                             *
 *                                                                             *
 *******************************************************************************/
- (unsigned long) calcSum;
- read:(NXTypedStream*)s;
- write:(NXTypedStream*)s;
- awake;
@end



